* -----------------------------------------------------------------------------       
       subroutine feyn(n,k1,maxmul,ico)
       
       include 'common_feyn.h'
       include 'common_print.h'
      
       integer ico(k1,18)
       integer icoo(:,:)
       integer ik(:)
       integer ipa(:)
       integer m(:),f(:),isani(:)
      
       allocatable icoo,ik,ipa,m,f,isani
       save ik,icoo,ipa,m,f,isani

c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - c
c                                                                     c
c             Feynman graph finding algorithm                         c
c                                                                     c
c                                                                     c
c    The subroutine redo results to ico(k1,1:18) elements             c
c    that have the following description :                            c
c                                                                     c
c    Take an example  from w- w+ -> w- w+ a a                         c
c    which has 35 subamplitudes coming out form 156 contributions     c
c    resulting to 192 Feynman graphs (unitary-gauge)                  c
c                                                                     c
c  156  5  62  33  35  1  44  56  34  28  2 33  2  4 33  3 0 1        c
c                                                                     c
c                                                                     c
c    1/18 : the routine that has been used to produce it  (5)         c
c    2/18 : the momentum i0  (62)                                     c
c    3/18 : the flavour  m0  (33)                                     c
c    4/18 : the rank of the subamplitude (35 ;the 35-th subamplitude) c
c    5/18 : the rank of the multiplicity (1 ;the 1st contribution)    c
c    6/18 : the max of the multiplicity (44; 44 contributions)        c
c    7/18 : the momentum i1 (56)                                      c
c    8/18 : the flavour  m1 (34)                                      c
c    9/18 : the rank of its subamplitude (28;it comes form the 28-th) c
c   10/18 : the momentum i2 (2)                                       c
c   11/18 : the flavour  m2 (33)                                      c
c   12/18 : the rank of its subamplitude (2;it comes form the 2-nd)   c
c   13/18 : the momentum i3 (4)                                       c
c   14/18 : the flavour  m3 (33)                                      c
c   15/18 : the rank of its subamplitude (4;it comes form the 4-th)   c
c   16/18 : the chirality (0); relevant for fermions only  (1/2)      c
c   17/18 : the isign (1); from fermions permutations                 c
c   18/18 : igluon                                                    c
c                                                                     c
c                                                                     c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - c
       
c      print*,'k1=',k1,ico(k1,4),ico(k1,6),maxmul,n
       
       nn=n

       
       allocate(icoo(0:ico(k1,4),0:maxmul),
     .          ik(0:ico(k1,4)),
     .          ipa(n),
     .          m(nn),
     .          f(nn),
     .          isani(2:2**(n-1)),
     .          stat=istat)

       if(istat.ne.0)then
        print*,'warning: allocation is not working properly in feyn'
        stop
       endif
        
       ipa(1:n)=0
       m(1:nn)=0
       f(1:nn)=0
       icoo(0:ico(k1,4),0:maxmul)=0
       ik(0:ico(k1,4))=1
       
c      print*,'maxmul=',maxmul
       if(maxmul.le.0)goto13
       k=1
       
       do i1=1,k1
       
       if(ico(k,5).eq.0)then
          ico(k:k1-1,1:18)=ico(k+1:k1,1:18)
          k=k-1
       endif
       k=k+1
       enddo
      
       k=k-1
       
       if(k.eq.0)goto13

       do j=1,k
        ico(j,5)=ico(j,6)-ico(j,5)+1
c       print'(i6,18i4)',j,ico(j,1:18)
        icoo(ico(j,4),ico(j,5))=j
       enddo
       
       
       ik(0:ico(k,4))=1
       
       j1=k
       ix=1
       l=1
       ixch=n-1
       
       
 11    continue
       
       ipa(ix)=icoo(ico(j1,4),ik(ico(j1,4)))
       goto12
       
 10    continue
       if(ixch.eq.0)goto13
c---------------------------------------------------------
c111   if(ipa(ix).eq.0)then
c       ix=ix-1
c       go to 111
c      endif
c---------------------------------------------------------

       j1=ipa(ix)

 12    continue

       if(ix.eq.ixch)then
       ik(ico(j1,4))=ik(ico(j1,4))+1
c---------------------------------------------------------
c      if(ik(ico(j1,4)).le.ico(j1,6))
c    . ipa(ix)=icoo(ico(j1,4),ik(ico(j1,4)))
c---------------------------------------------------------
       if(ik(ico(j1,4)).le.ico(j1,6))then
        ipa(ix)=icoo(ico(j1,4),ik(ico(j1,4)))
c       j1=ipa(ix)
       endif
       endif
       
c---------------------------------------------------------
c12    continue
c---------------------------------------------------------
 
       
       if(ik(ico(j1,4)).gt.ico(j1,6))then
        ik(ico(j1,4))=1
        ix=1
        ixch=ixch-1
        l=1
        f(1:nn)=0
        m(1:nn)=0
        goto10
       endif
       

       j1=icoo(ico(j1,4),ik(ico(j1,4)))
       iv=1
c---------------------------------------------------------
c      if(ico(j1,13).ne.0)iv=2
c---------------------------------------------------------
       m(l)  =ico(j1,7)
       f(l)  =ico(j1,9)
       m(l+1)=ico(j1,10)
       f(l+1)=ico(j1,12)
       m(l+2)=ico(j1,13)
       f(l+2)=ico(j1,15)
       
c      print*,l,m(l),f(l),m(l+1),f(l+1),m(l+2),f(l+2)
c      print*,ik(f(l+2))
c      print*,icoo(f(l+2),ik(f(l+2)))

       if(level(n,m(l+2)).le.1           !)then
     . .or.icoo(f(l+2),ik(f(l+2))).eq.0)then
        m(l+2:nn-1)=m(l+3:nn)
        m(nn)=0
        f(l+2:nn-1)=f(l+3:nn)
        f(nn)=0
c---------------------------------------------------------
c       m(l+2)=0
c       f(l+2)=0
c---------------------------------------------------------
       endif
       if(level(n,m(l+1)).le.1           !)then
     . .or.icoo(f(l+1),ik(f(l+1))).eq.0)then
        m(l+1:nn-1)=m(l+2:nn)
        m(nn)=0
        f(l+1:nn-1)=f(l+2:nn)
        f(nn)=0
c---------------------------------------------------------
c       m(l+1)=m(l+2)
c       f(l+1)=f(l+2)
c       m(l+2)=0
c       f(l+2)=0
c---------------------------------------------------------
       endif
       if(level(n,m(l)).le.1           !)then
     . .or.icoo(f(l),ik(f(l))).eq.0)then
        m(l:nn-1)=m(l+1:nn)
        m(nn)=0
        f(l:nn-1)=f(l+1:nn)
        f(nn)=0
c---------------------------------------------------------
c       m(l)=m(l+1)
c       f(l)=f(l+1)
c       m(l+1)=m(l+2)
c       f(l+1)=f(l+2)
c       m(l+2)=0
c       f(l+2)=0
c---------------------------------------------------------
       endif
     
       j=0
       istop=0
       do while(istop.eq.0)
        j=j+1
        if(m(j).eq.0)istop=1
       enddo
       l=j-1
       
c      print'(12i4)',ix,ixch,l,ipa
       if (l.gt.0)then
       ix=ix+iv
       if(iv.eq.2)ipa(ix-1)=0
       endif
       if(ix.le.0.or.ix.gt.n-1)print*,'warninginfeynman',ix
      

       if(l.eq.0)then
        l=1
        ng=ng+1
c       print*,'ng=',ng,ix,ixch
c       goto1001
        j=0
        i=0
	isani(2:2**(n-1))=0
 101    j=j+1
	i=i+1
        if(i.gt.ix)goto102
        if(ipa(i).ne.0)then
        if(ico(ipa(i),18).le.1)then  ! start
        is(ng,j,1:2)=ico(ipa(i),2:3)
        is(ng,j,3:4)=ico(ipa(i),7:8)
        is(ng,j,5:6)=ico(ipa(i),10:11)
        is(ng,j,7:8)=ico(ipa(i),13:14)
	if(level(n,is(ng,j,1)).eq.1)
     .isani(is(ng,j,1))=isani(is(ng,j,1))+1
	if(level(n,is(ng,j,3)).eq.1)
     .isani(is(ng,j,3))=isani(is(ng,j,3))+1
	if(level(n,is(ng,j,5)).eq.1)
     .isani(is(ng,j,5))=isani(is(ng,j,5))+1
	if(level(n,is(ng,j,7)).eq.1)
     .isani(is(ng,j,7))=isani(is(ng,j,7))+1
c       print'(12i4)',ng,j,i,ipa(i),is(ng,j,1:8)
        goto101
        endif                        ! end
	elseif(ipa(i).eq.0)then
	j=j-1
        goto101
	endif
 102    continue
        do il2=2,n
	if(isani(2**(il2-1)).ne.1)then
c        print*,'warninginfeyn',il2,ng,isani(2**(il2-1))
         is(ng,1:8,1:8)=0
         ng=ng-1
         ixch=ix
         ix=1
         f(1:nn)=0
         m(1:nn)=0
         goto10
        endif
	enddo
 1001   continue
        ixch=ix
        ix=1
        f(1:nn)=0
        m(1:nn)=0
        goto10
       endif
       
     
       j1=icoo(f(l),ik(f(l)))
       
      
       goto11
 
 
 13    write(nunit1,*)
     . '(feynman.f)the number of Feynman graphs is:',ng
      
       deallocate(icoo,ik,ipa,m,f,isani)
      
       return      
       end
